import 'dart:io';

import 'package:dio/dio.dart';
import 'package:dio_retry_fixed/dio_retry_fixed.dart';
import 'package:pedantic/pedantic.dart';
import 'package:test/test.dart';

void main() async {
  group('http requests', () {
    Dio dio;

    setUp(() async {
      dio = Dio();
      dio.interceptors.add(RetryInterceptor(
          dio: dio,
          options: RetryOptions(retries: 15),
          errorCallback: (e) {
            print('hit the error callback');
            print(e.response);
          }));
      var tryCount = 0;
      Stream<HttpRequest> server;
      try {
        unawaited(HttpServer.bind(InternetAddress.loopbackIPv4, 4044).then((server) async {
          await for (HttpRequest res in server) {
            print('server recieved a request');

            if (tryCount < 10) {
              res.response.statusCode = HttpStatus.badRequest;
              print('responded with an error');
              tryCount++;
            } else {
              await res.response.add([1, 2, 3, 4, 5, 6]);
            }
            await res.response.close();
          }
        }));
      } on Exception catch (ex) {
        print('couldnt bind to that port 4044, $ex');
      }
    });

    test('retry works', () async {
      for (int i = 0; i <= 10; i++) {
        print('attempt $i');
        await dio.request<void>("http://127.0.0.1:4044");
      }
    });
  });
}
